/*!
This file implements the MyEventSegment class, designed 
to read out a LeCroy 4300B FERA ADC.  

Tim Hoagland
1/19/05
s04.thoagland@wittenberg.edu
*/


#include "MyEventSegment.h"              
static char* pPacketVersion = "1.0";           
// Packet version -should be changed whenever major changes are made


//constructor set Packet details
MyEventSegment::MyEventSegment(unsigned short tag,int slot, int branch, int crate):    
  m_MyPacket(tag, string("My Packet"), string("Sample documented packet"),
	     string(pPacketVersion))
{
  Tag=tag;             // Packet Id #
  Slot = slot;         //CAMAC slot #
  Branch = branch;     //CAMAC branch #
  Crate = crate;       //CAMAC crate #
}


// Is called right after the module is created.  All one time Setup
// should be done now.
void MyEventSegment::Initialize()
{
  int userints[16];    // array of pedastal values
  for ( int i=0; i<16; i++)  //sets all array elements to 1
    {
      userints[i] = 1;
    }
  branchinit(Branch);   // initializes the branch controller
  crateinit(Branch, Crate); //initializes crate
  CLRFERA(Branch, Crate, Slot);  // clears module

  //initializes module, sets control register bits
  INITFERA(Branch,Crate, Slot, FERA_CLE | FERA_CCE , 0) ; 
}


// Is called after reading data buffer
void MyEventSegment::Clear()
{
  CLRFERA(Branch, Crate, Slot);// clears module
}


unsigned int MyEventSegment::MaxSize()
{
  return 16*4+2;  
}

 
//Is called to readout data on module
DAQWordBufferPtr& MyEventSegment::Read(DAQWordBufferPtr& bufpt)
{ 
  if (qtst(Branch) ==0) return bufpt;  // tests if data present
  bufpt = m_MyPacket.Begin(bufpt);  //opens new packet

  for (int i=0; i<16; i++)  // loops though all channels
   {
     *bufpt=i;   // writes channel #
     ++bufpt;
     *bufpt = 0x7ff & READFERA(Branch, Crate, Slot, i);
                                 // writes ADC Value

   }
  CLRFERA(Branch, Crate, Slot); // clears module
  m_MyPacket.End(bufpt); //closes Packet
  return bufpt; 
}
