/*!
This is the implenetation file for MyEventProcessor.  It is
intended to be used to decode a packet from a 
LeCroy 4300B FERA ADC.

Tim Hoagland
1/19/05
s04.thoagland@wittenberg.edu
*/

#include "MyEventProcessor.h"
#include <TCLAnalyzer.h>
#include <Event.h>
#include <FilterBufferDecoder.h>


// Constuctor: defines the Id to decode and Base slot to 
// put data into
MyEventProcessor::MyEventProcessor (int base, int Id)  
{
  nOurId = Id;  // Id to unpack
  Base = base;  //Base slot # to store data
}  



// Searches for the Id tag we want, asks to have it unpacked if
// the right tag is found.
Bool_t MyEventProcessor::operator()(const Address_t pEvent, CEvent& rEvent,
			 CAnalyzer& rAnalyzer, CBufferDecoder& rDecoder)
{
  TranslatorPointer<UShort_t> p(*(rDecoder.getBufferTranslator()), pEvent);
  CTclAnalyzer& rAna((CTclAnalyzer&)rAnalyzer);

  UShort_t nWords=*p++;           // get number of words in the packet
  rAna.SetEventSize(nWords*sizeof(UShort_t)); 
  nWords--;

  while(nWords)                  // Search packet for ID tag
    {
      UShort_t nPacketWords=*p;
      UShort_t nId = p[1];
      if(nId == nOurId)
	{
	  UnpackPacket(p, rEvent);  // if the right Id tag is found 
	}                           // Unpack the packet
      p +=nPacketWords;
      nWords -= nPacketWords;
    }
  return kfTRUE;
}


void MyEventProcessor::UnpackPacket(TranslatorPointer<UShort_t> pEvent,
				    CEvent& rEvent)
{
  UInt_t nPacketSize = *pEvent;  // get number of words let to be read
  ++pEvent;++pEvent;             // skip over Id info and Header words
  nPacketSize -=2;               // adjust words remaining


  while(nPacketSize)  
    {
      UInt_t nChannel = *pEvent;  // get the channel number
      ++pEvent;                   // look at next word
      unsigned int value = *pEvent; // get ADC value
      rEvent[Base + nChannel] = value;  // record ADC value
      ++pEvent;   // increment pointer
      nPacketSize -=2;    // subtract from packet size

      // Below this line is a test line that can be uncommented to look at raw data
      //cerr << "found a value of " <<value << " at channel " << nChannel <<endl;
    }
}
